<?php

declare(strict_types=1);

namespace Drupal\jsld\Hook\Entity;

use Drupal\Component\Serialization\Json;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\jsld\Plugin\jsld\JsldPluginInterface;
use Drupal\jsld\Plugin\jsld\JsldPluginManagerInterface;
use Drupal\jsld\Utility\EntityMatcher;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides hook entity view preprocess.
 *
 * @see \jsld_entity_view()
 */
final readonly class EntityView implements ContainerInjectionInterface {

  /**
   * Constructs a new EntityView object.
   */
  public function __construct(
    protected JsldPluginManagerInterface $entityJsldPluginManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('plugin.manager.jsld.entity'),
    );
  }

  /**
   * Implements hook_entity_view().
   */
  public function __invoke(array &$build, EntityInterface $entity, EntityViewDisplayInterface $display, string $view_mode): void {
    foreach ($this->entityJsldPluginManager->getDefinitions() as $plugin_id => $plugin) {
      if ($plugin['entity_type'] !== $entity->getEntityTypeId()) {
        continue;
      }

      if (!EntityMatcher::matchBundleViewMode($entity->bundle(), $view_mode, $plugin['entity_limit'])) {
        continue;
      }

      $instance = $this->entityJsldPluginManager->createInstance($plugin_id, [
        'entity' => $entity,
        'view_mode' => $view_mode,
      ]);
      \assert($instance instanceof JsldPluginInterface);

      if (!$instance->isEnabled()) {
        continue;
      }

      $entity_type_id = $entity->getEntityTypeId();
      $bundle = $entity->bundle();
      $entity_id = $entity->id();
      $build['#attached']['html_head'][] = [
        [
          '#type' => 'html_tag',
          '#tag' => 'script',
          '#attributes' => [
            'type' => 'application/ld+json',
          ],
          '#value' => Json::encode($instance->build()),
        ],
        // It must be unique, otherwise all duplicate IDs will be filtered out
        // by Drupal. Entity plugins must be able to add more than one JSON-LD
        // to the page (e.g., list of teasers — each teaser must be able to add
        // its own JSON-LD).
        "jsld_{$plugin_id}:{$entity_type_id}:{$bundle}:{$entity_id}:{$view_mode}",
      ];
    }
  }

}
